(function (factory) {
    if (typeof define === "function" && define.amd) {
        define([
            "jquery",
            "jquery/ui"
        ], factory);
    } else {
        factory(jQuery);
    }
}(function ($) {
    "use strict";

    $.widget('neklo.timeline', {
        options: {
            stride : 60*60*1000 //in milisecconds
        },

        /**
         * @private
         */
        _create: function()
        {
            this.jobs = {};
            this.tooltip;
            this.limits = [
                this.options.timeStart * 1000,
                this.options.timeEnd * 1000
            ];
            if (this.limits[0] > this.limits[1]) {
                var date  = this.limits[0];
                this.limits[0] = this.limits[1];
                this.limits[1] = date;
            }
            this._prepareData();
            this._initTimeline();
            this._initTooltip();
        },

        _initTooltip : function()
        {
            this.tooltip = $('<div class="timeline-tooltip" style="display:none;">');
            $('body').append(this.tooltip);
            var that = this;
            this.tooltip.on('mouseover', function(){
                var timeout = that.tooltip.attr('data-hide');
                //console.log(timeout);
                if (timeout) {
                    clearTimeout(timeout*1);
                    that.tooltip.removeAttr('data-hide');
                }
            }).on('mouseout', function(event) {
                var tg = (window.event) ? event.srcElement : event.target;
                var reltg = (event.relatedTarget) ? event.relatedTarget : event.toElement;
                if (!$(tg).hasClass('timeline-tooltip')) return;
                while (reltg != tg && reltg.nodeName != 'BODY')
                    reltg= reltg.parentNode;
                if (reltg == tg) return;
                // Mouseout took place when mouse actually left layer
                // Handle event
                that.tooltip.hide();
            });
        },

        /**
         * @private
         */
        _initTimeline: function()
        {
            var hour = this.element.find('.hour');
            var hourPanel = hour.parent();
            hour.remove();
            hour.removeClass('template');
            var date;
            var timeSpan = (this.limits[1] - this.limits[0]);
            var slots = Math.ceil(timeSpan / this.options.stride);
            var min, mon;
            for(var i = this.limits[0]; i <= this.limits[1]; i += this.options.stride) {
                date = new Date(i);
                min = date.getMinutes();
                mon = date.getMonth()+1;
                min = min < 10 ? '0'+min : min;
                mon = mon < 10 ? '0'+mon : mon;
                hour.css('width', (100/slots)+'%')
                    .find('span').html(
                        date.getDate()+'.'+mon+'.<br>'+
                        date.getFullYear()+'<br>'+
                        date.getHours()+':'+min
                        );
                hourPanel.append(hour.clone());
            }
            var width = hourPanel.width();
            var jobName = this.element.find('.jobs .job-mane');
            var jobData = this.element.find('.timeline-container .job-data');
            var nameParent = jobName.parent();
            var dataParent = jobData.parent();
            jobName.remove();
            jobData.remove();
            for(var i in this.jobs) {
                var tpl = jobData.clone();
                jobName.html(i);
                //jobData.html(i);
                tpl.width(width);
                tpl.attr('data-code', i);
                nameParent.append(jobName.clone());
                dataParent.append(tpl);
                this._addData(tpl, this.jobs[i], timeSpan);
            }
        },

        _addData: function(jobElement, data, timeSpan)
        {
            var that = this;
            var eventTpl = jobElement.find('.event');
            var width = timeSpan/jobElement.width();//time per 1px
            eventTpl.remove();
            var event, start, stop, toStart, last ,w;
            var times = [];
            var counts = [];
            var idx;
            for(var i = 0; i < data.length; i++) {
                var tpl = eventTpl.clone(true);
                event = data[i];
                start = Date.parse(event.executed_at);
                stop = Date.parse(event.finished_at);
                toStart = Date.parse(event.scheduled_at);
                idx = this._isThere(times, toStart, 4*width);
                if (idx < 0){
                    times.push(toStart);
                    counts.push(0);
                    idx = 0;
                } else {
                    counts[idx]++;
                }
                jobElement.append(tpl);
                for(var name in event) {
                    tpl.find('.process .'+name+' span').html(event[name]);
                }
                last = (toStart - this.limits[0])/(timeSpan/100);
                //console.log(last);
                tpl.css('left', last+'%')
                    .css('height', 39-(counts[idx]*4)+'px')
                    .css('top', (counts[idx]*4)+'px')
                    .addClass(event.status);
                if (i > 0) {
                    tpl.css('border-top-width', '1px')
                }
                w = (stop - start)/width;
                if(w < 1) w = 1;
                if (!isNaN(start) && !isNaN(stop)) {
                    tpl.find('.process-bar')
                        .css('height', 39-(counts[idx]*4)+'px')
                        .css('margin-left', ((start - toStart)/width)+'px')
                        .css('width', w+'px');
                } else {
                    if (event.status !== 'error') {
                        tpl.find('.process-bar').hide();
                    }
                }
                tpl.bind('mouseenter', function(event){
                    that.showToolTip(this, event);
                }).bind('mouseout', function(event){
                    var tg = (window.event) ? event.srcElement : event.target;
                    var reltg = (event.relatedTarget) ? event.relatedTarget : event.toElement;
                    if (!$(tg).hasClass('event')) return;
                    while (reltg != tg && reltg.nodeName != 'BODY')
                        reltg= reltg.parentNode;
                    if (reltg == tg) return;
                    that.hideToolTip(this);
                });
            }
        },
        /**
         * checks, if there is timestamp within margin and returns index. -1 default
         * @returns index
         */
        _isThere: function(stampArray, stamp, margin)
        {
            var half1 = stamp - (margin/2);
            var half2 = stamp + (margin/2);
            for(var i in stampArray) {
                if (stampArray[i] >= half1 && stampArray[i] <= half2 ) {
                    return i;
                }
            }
            return -1;
        },

        showToolTip : function(element, event)
        {
            var el = $(element);
            //console.log(this.tooltip.attr('data-hide'));
            if (this.tooltip.attr('data-hide')) {
                clearTimeout(this.tooltip.attr('data-hide')*1);
                this.tooltip.removeAttr('data-hide');
            }
            this.tooltip.html(el.find('.process')[0].outerHTML);
            this.tooltip.css('top', (event.pageY+5)+'px');
            this.tooltip.css('left', (event.pageX+5)+'px');
            this.tooltip.show();
        },
        hideToolTip : function(element)
        {
            var that = this;
            var timeout = setTimeout(function(){
                that.tooltip.hide();
            }, 1000);
            this.tooltip.attr('data-hide', timeout);
        },

        _prepareData: function()
        {
            var item;
            var that = this;
            var minmax = function(value){
                if (!isNaN(value)) {
                    if (value < that.limits[0]) {
                        that.limits[0] = value;
                    } else if(value > that.limits[1]) {
                        that.limits[1] = value;
                    }
                }
            }
            for(var i in this.options.dataSource) {
                item = this.options.dataSource[i];
                if (!this.jobs[item.job_code]) {
                    this.jobs[item.job_code] = [];
                }
                this.jobs[item.job_code].push(item);
                minmax(Date.parse(item.created_at));
                minmax(Date.parse(item.executed_at));
                minmax(Date.parse(item.finished_at));
                minmax(Date.parse(item.scheduled_at));
            }
            this.limits[0] -= this.options.stride;
            this.limits[1] += this.options.stride;
        }
    });

    return $.neklo.timeline;
}));